var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var __privateMethod = (obj, member, method) => {
  __accessCheck(obj, member, "access private method");
  return method;
};
var _manifest, _manifestData, _rootFolder, _routeDataToRouteInfo, _routeCache, _renderersPromise, _loadRenderers, loadRenderers_fn, _loadModule, loadModule_fn;
import { defaultLogOptions } from "../logger.js";
import { matchRoute } from "../routing/match.js";
import { render } from "../render/core.js";
import { RouteCache } from "../render/route-cache.js";
import { createLinkStylesheetElementSet, createModuleScriptElementWithSrcSet } from "../render/ssr-element.js";
import { createRenderer } from "../render/renderer.js";
import { prependForwardSlash } from "../path.js";
class App {
  constructor(manifest, rootFolder) {
    __privateAdd(this, _loadRenderers);
    __privateAdd(this, _loadModule);
    __privateAdd(this, _manifest, void 0);
    __privateAdd(this, _manifestData, void 0);
    __privateAdd(this, _rootFolder, void 0);
    __privateAdd(this, _routeDataToRouteInfo, void 0);
    __privateAdd(this, _routeCache, void 0);
    __privateAdd(this, _renderersPromise, void 0);
    __privateSet(this, _manifest, manifest);
    __privateSet(this, _manifestData, {
      routes: manifest.routes.map((route) => route.routeData)
    });
    __privateSet(this, _rootFolder, rootFolder);
    __privateSet(this, _routeDataToRouteInfo, new Map(manifest.routes.map((route) => [route.routeData, route])));
    __privateSet(this, _routeCache, new RouteCache(defaultLogOptions));
    __privateSet(this, _renderersPromise, __privateMethod(this, _loadRenderers, loadRenderers_fn).call(this));
  }
  match({ pathname }) {
    return matchRoute(pathname, __privateGet(this, _manifestData));
  }
  async render(url, routeData) {
    if (!routeData) {
      routeData = this.match(url);
      if (!routeData) {
        return "Not found";
      }
    }
    const manifest = __privateGet(this, _manifest);
    const info = __privateGet(this, _routeDataToRouteInfo).get(routeData);
    const [mod, renderers] = await Promise.all([__privateMethod(this, _loadModule, loadModule_fn).call(this, info.file), __privateGet(this, _renderersPromise)]);
    const links = createLinkStylesheetElementSet(info.links, manifest.site);
    const scripts = createModuleScriptElementWithSrcSet(info.scripts, manifest.site);
    return render({
      legacyBuild: false,
      links,
      logging: defaultLogOptions,
      markdownRender: manifest.markdown.render,
      mod,
      origin: url.origin,
      pathname: url.pathname,
      scripts,
      renderers,
      async resolve(specifier) {
        if (!(specifier in manifest.entryModules)) {
          throw new Error(`Unable to resolve [${specifier}]`);
        }
        const bundlePath = manifest.entryModules[specifier];
        return prependForwardSlash(bundlePath);
      },
      route: routeData,
      routeCache: __privateGet(this, _routeCache),
      site: __privateGet(this, _manifest).site
    });
  }
}
_manifest = new WeakMap();
_manifestData = new WeakMap();
_rootFolder = new WeakMap();
_routeDataToRouteInfo = new WeakMap();
_routeCache = new WeakMap();
_renderersPromise = new WeakMap();
_loadRenderers = new WeakSet();
loadRenderers_fn = async function() {
  const rendererNames = __privateGet(this, _manifest).renderers;
  return await Promise.all(rendererNames.map(async (rendererName) => {
    return createRenderer(rendererName, {
      renderer(name) {
        return import(name);
      },
      server(entry) {
        return import(entry);
      }
    });
  }));
};
_loadModule = new WeakSet();
loadModule_fn = async function(rootRelativePath) {
  let modUrl = new URL(rootRelativePath, __privateGet(this, _rootFolder)).toString();
  let mod;
  try {
    mod = await import(modUrl);
    return mod;
  } catch (err) {
    throw new Error(`Unable to import ${modUrl}. Does this file exist?`);
  }
};
export {
  App
};
