import * as fs from "fs";
import { App } from "./index.js";
import { deserializeManifest } from "./common.js";
function createURLFromRequest(req) {
  return new URL(`http://${req.headers.host}${req.url}`);
}
class NodeApp extends App {
  match(req) {
    return super.match(req instanceof URL ? req : createURLFromRequest(req));
  }
  render(req) {
    return super.render(req instanceof URL ? req : createURLFromRequest(req));
  }
}
async function loadManifest(rootFolder) {
  const manifestFile = new URL("./manifest.json", rootFolder);
  const rawManifest = await fs.promises.readFile(manifestFile, "utf-8");
  const serializedManifest = JSON.parse(rawManifest);
  return deserializeManifest(serializedManifest);
}
async function loadApp(rootFolder) {
  const manifest = await loadManifest(rootFolder);
  return new NodeApp(manifest, rootFolder);
}
export {
  loadApp,
  loadManifest
};
