import path from "path";
import { viteID } from "../../util.js";
const STYLE_EXTENSIONS = /* @__PURE__ */ new Set([".css", ".pcss", ".postcss", ".scss", ".sass", ".styl", ".stylus", ".less"]);
const cssRe = new RegExp(`\\.(${Array.from(STYLE_EXTENSIONS).map((s) => s.slice(1)).join("|")})($|\\?)`);
const isCSSRequest = (request) => cssRe.test(request);
function getStylesForURL(filePath, viteServer) {
  const css = /* @__PURE__ */ new Set();
  function crawlCSS(id, scanned = /* @__PURE__ */ new Set()) {
    const relatedMods = viteServer.moduleGraph.getModulesByFile(id) ?? /* @__PURE__ */ new Set();
    const importedModules = /* @__PURE__ */ new Set();
    for (const relatedMod of relatedMods) {
      if (id === relatedMod.id) {
        scanned.add(id);
        for (const importedMod of relatedMod.importedModules) {
          importedModules.add(importedMod);
        }
      }
    }
    for (const importedModule of importedModules) {
      if (!importedModule.id || scanned.has(importedModule.id))
        continue;
      const ext = path.extname(importedModule.url.toLowerCase());
      if (STYLE_EXTENSIONS.has(ext)) {
        css.add(importedModule.url);
      }
      crawlCSS(importedModule.id, scanned);
    }
  }
  crawlCSS(viteID(filePath));
  return css;
}
export {
  STYLE_EXTENSIONS,
  getStylesForURL,
  isCSSRequest
};
